function [ POINTS ] = interp3d(DIRECTORY, OUTPUTFILE, SPACING, UPSAMPLE, OUTPUTDIR, COLOR)
% [ POINTS ] = interp3d(DIRECTORY, OUTPUTFILE, SPACING, UPSAMPLE, OUTPUTDIR, COLOR)
%
% Interpolate planar outlines to create a three-dimensional solid.
%
% This is an older version of ras2sld; this version also reads the OTL files, and saves the 
% result in an output file.  It is provided for user convenience, but is no longer used by
% the mrcat GUI.
%
% Output:
% POINTS (3xNPTS)     - X, Y, and Z coordinates of each point on the 3D interpolated outline.
%
% Required Inputs:
% DIRECTORY (string)  - directory from which to read .otl files
% SPACING (scalar)    - spacing between image slices, in pixel units
%
% Optional Inputs:
% OUTPUTFILE (string) - This file contains a copy of the variable POINTS,
%                       written using otlwrite.m, in three dimensions.
%                       (if OUTPUTFILE is '', POINTS is not written out to a file)
%
% UPSAMPLE (scalar)   - factor by which X,Y,Z should be upsampled (default: 2)
% OUTPUTDIR (string)  - if specified, distance images are written to this directory
%                       (if OUTPUTDIR is '', distance images are not written out)
% COLOR (char)        - color code ('g', 'b', etc) of the outline to be interpreted
%                       default value ('') means to interpolate all points as a single outline
%
% The outline files in DIRECTORY are treated as a series of slices,
% sorted in ASCII-sort order, and with an inter-slice spacing of SPACING
% (given in the same units as the outline files, i.e. usually given in
% pixels.  SPACING need not be an integer).
%
% The outlines are interpolated using Raya and Udupa's "shape-based
% interpolation" algorithm (IEEE Trans. on Medical Imaging, 9(1):32-42, 1990).
% The distance-map calculation is performed using the routine otl2dmap.m.
% All (X,Y) coordinates are multiplied by a factor of UPSAMPLE prior to
% interpolation, and we create ceil(UPSAMPLE*SPACING)-1 interpolated slices
% between each given slice.
%
% The resulting solid object is converted to a surface by finding all
% voxel faces with a negative distance on one side and a non-negative distance
% on the other side.  The (X,Y,Z) coordinates of the centerpoint of each such
% face are multiplied by a scaling factor, rounded to the nearest integer, and stored
% in OUTPUTFILE as a three-channel short integer stream with a 1024-byte 
% NIST_1A header.
%
% If OUTPUTDIR is specified, distance images will also be stored in
% OUTPUTDIR in GE 5.x file format (because it's a 16-bit format, that's why)
% in files of the form OUTPUTDIR/IMG_NUM.mr, where IMG is the image
% we're interpolating from, and NUM (between 0 and ceil(SPACING)-1) tells
% you how far along we are between IMG and the next image.
%
% Version 1.2, Mark Hasegawa-Johnson, 6/2/99
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step 1:
%  Check the arguments, and open various files and directories
%

% Define UPSAMPLE
if nargin<4, UPSAMPLE=2; end
if nargin<5, OUTPUTDIR=[]; end
if nargin<6, COLOR=''; end

% Initialize the output POINTS array to have the correct number of rows
POINTS = zeros(3,0);

% Find out whether or not we should write out DMAP images
write_dmaps = 0;
if length(OUTPUTDIR)>0,
  if length(dir(OUTPUTDIR)) > 0, 
    write_dmaps = 1; 
  else,
    disp(sprintf('interp3d will not write DMAPs to a nonexistant directory: %s',OUTPUTDIR));
  end;
end

% Get an ASCII-sorted listing of the .otl files in the input directory
[ AllStrings, Files, Dirs ] = filemenu(DIRECTORY, { 'otl' } );
if length(Files)==0,
  error(sprintf('interp3d: Unable to find any .otl files in %s', DIRECTORY));
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step 2:
%   Read in the highest-numbered OTL file, create the first DMAP,
%   and initialize the list of output POINTS

% Read in the highest-numbered OTL file
istart = length(Files)-1;
maxZ = istart * SPACING;
[STARTX, HIN] = otlread(fullfile(DIRECTORY, Files{istart+1}));
if length(COLOR)>0 & HIN.channel_count<3,
  STARTX = [STARTX; repmat(double('g'), 1, HIN.sample_count) ];
end
[ STARTX, COLORS ] = otl2ras(STARTX, HIN);

% Choose points of the desired color
if length(COLOR)>0,
  which_points = find(floor(STARTX(3,:)) == double(COLOR(1)));
  STARTX = STARTX(:, which_points);
end

% Check to make sure we have non-zero outline size
if prod(size(STARTX)) > 0,
  % Create an ENDMAP
  maxX = ceil(max(STARTX')) + 1;
  minX = floor(min(STARTX')) - 1;
  mapsize = [ maxX(2)-minX(2), maxX(1)-minX(1) ];
  ENDMAP = otl2dmap(UPSAMPLE*(STARTX(1,:)-minX(1)), UPSAMPLE*(STARTX(2,:)-minX(2)), UPSAMPLE*mapsize, UPSAMPLE*2 );
  
  % Find all zero-crossings inside the first plane
  [X, Y] = zerocrossings(ENDMAP);
  Z = maxZ - istart * SPACING;
  
  % Add these points to the output POINTS array
  POINTS = [ POINTS, [X/UPSAMPLE + minX(1); Y/UPSAMPLE + minX(2); repmat(Z, 1, length(X)) ]];
  
  % Write out the first DMAP to a GE image file, if so desired
  if write_dmaps,
    gehead = struct('mri', struct('corners', zeros(4,3), 'user', zeros(1,27)));
    gehead.mri.corners(:,1:2) = [ minX([2 1]); maxX(2) minX(1); minX(2) maxX(1);  maxX([2 1]) ];
    gehead.mri.corners(:,3) = repmat(Z, [4 1]);
    gefilename = fullfile(OUTPUTDIR, sprintf('%s_00.mr', strtok(Files{istart+1}, '.')));
    bytes = dmapwr(ENDMAP, gehead, gefilename);
  end;
else,
  
end
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step 3:
%   Work our way down through Z, reading in otl files, and creating
%   all interpolations between each pair of otl files.
%
for istart=(length(Files)-1):-1:1,
  % ENDX is the previous iteration's value of STARTX, oldZ is the previous value of Z
  ENDX = STARTX;
  oldZ = maxZ - istart * SPACING;
  
  % Read in the next outline file
  [ STARTX, HIN ] = otlread(fullfile(DIRECTORY, Files{istart}));
  if length(COLOR)>0 & HIN.channel_count<3,
    STARTX = [STARTX; repmat(double('g'), 1, HIN.sample_count) ];
  end
  [ STARTX, COLORS ] = otl2ras(STARTX, HIN);
  disp(sprintf('Processing OTL file %s', fullfile(DIRECTORY, Files{istart})))
  
  % Choose points of the desired color
  if length(COLOR)>0,
    which_points = find(floor(STARTX(3,:)) == double(COLOR(1)));
    STARTX = STARTX(:, which_points);
  end

  % Check to make sure that both of the maps has non-zero outline length
  if all( [size(STARTX) size(ENDX)]>0 ),
    
    % Find the range of x and y in which all boundaries of the interpolated images will fall
    maxX = ceil(max([STARTX(1:2,:)'; ENDX(1:2,:)'])) + 1;
    minX = floor(min([STARTX(1:2,:)'; ENDX(1:2,:)'])) - 1;
    mapsize = [ maxX(2)-minX(2), maxX(1)-minX(1) ];
    
    % Create the STARTMAP and ENDMAP in the range given by minX and maxX
    STARTMAP = otl2dmap(UPSAMPLE*(STARTX(1,:)-minX(1)), UPSAMPLE*(STARTX(2,:)-minX(2)), UPSAMPLE*mapsize, UPSAMPLE*2 );
    ENDMAP = otl2dmap(UPSAMPLE*(ENDX(1,:)-minX(1)), UPSAMPLE*(ENDX(2,:)-minX(2)), UPSAMPLE*mapsize, UPSAMPLE*2 );
    
    % OLDMAP starts out equal to the ENDMAP
    OLDMAP = ENDMAP;
    
    % Create each interpolated layer
    % This begins by creating the layer just below ENDMAP
    % The last layer created (the num==0 case) is actually equal to STARTMAP;
    % by putting it here, we only have to write the output commands once.
    for num=(ceil(UPSAMPLE*SPACING)-1):-1:0,
      frac = num / (UPSAMPLE*SPACING);
      Z = maxZ - (istart-1+frac) * SPACING;

      % Create the interpolated image, and save it if desired
      INTERMAP = (1-frac) * STARTMAP + frac * ENDMAP;
      if write_dmaps, 
	gehead.mri.corners(:,1:2) = [ minX([2 1]); maxX(2) minX(1); minX(2) maxX(1);  maxX([2 1]) ];
	gehead.mri.corners(:,3) = repmat(Z, [4 1]);
	gefilename = fullfile(OUTPUTDIR, sprintf('%s_%2.2d.mr', strtok(Files{istart}, '.'), num));
	bytes = dmapwr(INTERMAP, gehead, gefilename); 
      end;
      
      % Find all zero-crossings between OLDMAP and INTERMAP, and add them to POINTS
      [Y, X] = find(xor(INTERMAP<0, OLDMAP<0));
      nxy = length(X);
      newpts = [ X'/UPSAMPLE + minX(1); Y'/UPSAMPLE + minX(2); repmat((Z+oldZ)/2, [1 nxy]) ];
      POINTS = [ POINTS newpts ];
      
      % Find all zero-crossings inside INTERMAP, and write them to the output file
      [ X, Y ] = zerocrossings(INTERMAP);
      newpts = [ X/UPSAMPLE + minX(1); Y/UPSAMPLE + minX(2); repmat(Z, [1 length(X)]) ];
      POINTS = [ POINTS newpts ];
      
      % Create OLDMAP and oldZ for the next loop
      OLDMAP = INTERMAP;  
      oldZ = Z;
    end;
  end
end  
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step 4:
%   Write out the output points to the output outline file
%   That's all, folks!

if length(OUTPUTFILE)>0,
  otlwrite(OUTPUTFILE, [ POINTS ] );
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Internal function zerocrossings
% Find all edges within a plane where the intensity changes from below zero to above zero
function [X, Y] = zerocrossings(Im)

[maxY,maxX] = size(Im);

% B is one where Im >=0; B0 is the inverse of B
B = (Im >= 0);
B0 = (B==0);

% find all points such that B(Y1,X1)==1, but B(Y1,X1+1)==0
% .. the outline point is at Y1, X1+0.5
[Y1,X1] = find( B & [B0(:, 2:maxX) zeros(maxY,1)]);
X1 = X1+0.5;

% find all points such that B(Y2,X2)==1, but B(Y2, X2-1)==0
% .. the outline point is at Y2, X2-0.5
[Y2,X2] = find( B & [zeros(maxY,1) B0(:, 1:(maxX-1))]);
X2 = X2-0.5;

% find all points such that B(Y3,X3)==1, but B(Y3+1,X3)==0
% .. the outline point is at Y3+0.5, X3
[Y3,X3] = find( B & [B0(2:maxY, :); zeros(1,maxX)]);
Y3 = Y3+0.5;		 

% find all points such that B(Y3,X3)==1, but B(Y3-1,X3)==0
% .. the outline point is at Y4-0.5, X4
[Y4,X4] = find( B & [zeros(1,maxX); B0(1:(maxY-1), :)] );
Y4 = Y4-0.5;

% Sort together all of these points, and remove duplications
XY = unique([ X1 Y1; X2 Y2; X3 Y3; X4 Y4 ], 'rows' );
if prod(size(XY))==0, XY=zeros(0,2); end
X = XY(:,1)';
Y = XY(:,2)';

